import java.util.List;
import java.util.ArrayList;

/**
 * A composite callback that distributes data
 * to all registered callbacks for functionality B.
 *
 * @author Tom Verhoeff (Eindhoven University of Technology)
 */
public class CompositeCallbackB implements CallbackB {
    
    /** List of registered callbacks */
    private final List<CallbackB> parts;
    
    /**
     * Constructs an empty composite callback.
     */
    public CompositeCallbackB() {
        parts = new ArrayList<CallbackB>();
    }
    
    /**
     * Registers a callback.
     * 
     * @param b  callback for functionality B to register
     * @pre {@code b != null}
     * @throws IllegalArgumentException  if {@code b == null}
     */
    public void add(CallbackB b) {
        if (b == null) {
            throw new IllegalArgumentException(
                    "CompositeCallbackB.add: pre b != null failed");
        }
        parts.add(b);
    }
    
    /**
     * Unregisters a callback.
     * 
     * @param b  callback to unregister
     * @post {@code b} is unregistered, if it was registered
     */
    public void remove(CallbackB b) {
        parts.remove(b);
    }
    
    /**
     * Distributes data to all registered callbacks.
     */
    @Override
    public void doB(String data) {
        for (CallbackB b : parts) {
            b.doB(data);
        }
    }
    
}
