import junit.framework.TestCase;

/**
 * Smoke tests for composite callbacks.
 */
public class CompositeCallbackBTest extends TestCase {
    
    /**
     * Tests doB() for composite with no registered callbacks.
     * Checks that nothing goes wrong.
     */
    public void testAdd0() {
        final CompositeCallbackB SUT = new CompositeCallbackB();
        SUT.doB("test");
        assertTrue(true);
    }
    
    /**
     * Tests add() and doB() for composite with one registered callback.
     * Checks that a data item is passed on.
     */
    public void testAdd1() {
        final CompositeCallbackB SUT = new CompositeCallbackB();
        final String expected = "test";
        class Flag {
            public boolean state = false;
        };
        final Flag called = new Flag();
        // set up anonymous callback that checks passed data:
        final CallbackB callback = new CallbackB() {
            @Override
            public void doB(String data) {
                assertEquals("doB", expected, data);
                called.state = true;
                // N.B. cannot change local variable of enclosing method
            }
        };
        SUT.add(callback);
        SUT.doB(expected);
        assertTrue("Callback called", called.state);
    }
    
    /**
     * Tests add() and doB() for composite with two registered callbacks.
     */
    public void testAdd2() {
        final CompositeCallbackB SUT = new CompositeCallbackB();
        final String expected = "test";
        class MyCallbackB implements CallbackB {
            public boolean called = false;
            @Override
            public void doB(String data) {
                assertEquals("doB", expected, data);
                called = true;
            }
        };
        final MyCallbackB callback1 = new MyCallbackB();
        final MyCallbackB callback2 = new MyCallbackB();
        SUT.add(callback1);
        SUT.add(callback2);
        SUT.doB(expected);
        assertTrue("Callback1 called", callback1.called);
        assertTrue("Callback2 called", callback2.called);
    }
    
    /**
     * Tests add() for IllegalArgumentException.
     */
    public void testAddException() {
        final CompositeCallbackB SUT = new CompositeCallbackB();
        try {
            SUT.add(null);
            fail("Expected IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            assertTrue(true);
        } catch (Exception e) {
            fail("Expected IllegalArgumentException, but got " + e);
        }
    }
    
    /**
     * Tests remove() with non-registered callback.
     */
    public void testRemove0() {
        final CompositeCallbackB SUT = new CompositeCallbackB();
        final CallbackB callback = new CallbackB() {
            @Override
            public void doB(String data) { }
        };
        SUT.remove(callback);
        assertTrue(true);
    }
    
    /**
     * Tests remove() with registered callback.
     * Checks that removed callback is not triggered.
     */
    public void testRemove1() {
        final CompositeCallbackB SUT = new CompositeCallbackB();
        final CallbackB callback = new CallbackB() {
            @Override
            public void doB(String data) {
                fail("doB");
            }
        };
        SUT.add(callback);
        SUT.remove(callback);
        SUT.doB("test");
        assertTrue(true);
    }
    
}
